package edu.berkeley.edu.cs160.groupjybz.realdealprototype;

import java.io.FileNotFoundException;
import java.util.Random;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.widget.Toast;

/**
 * Simple notes database access helper class. Defines the basic CRUD operations
 * for the notepad example, and gives the ability to list all notes as well as
 * retrieve or modify a specific note.
 * 
 * This has been improved from the first version of this tutorial through the addition
 * of better error handling and also using returning a Cursor instead of using
 * a collection of inner classes (which is less scalable and not recommended).
 */
public class RealDealDbAdapter 
{
    public static final String KEY_TYPE="type";
    public static final String KEY_DIS="dis";
    public static final String KEY_KEYWORD="key_word";
    public static final String KEY_DESCRIPTION="description";
    public static final String KEY_LOCATION="location";
    public static final String KEY_STORENAME="store_name";
    public static final String KEY_UPVOTE="up_vote";
    public static final String KEY_TOTALVOTE="total_vote";
    public static final String KEY_ROWID="_id";
    public static final String KEY_VOTERATIO="vote_ratio";
    public static final String KEY_LAT="lat";
    public static final String KEY_LONGI="longi";
    public static final String KEY_RADIUS="radius";
    public static final String KEY_PROMOTION_PER_STORE="promotion_per_store";
    
    private Random generator = new Random();
    /**
     * Database creation sql statement
     * the dis value is set to a constant and stored in the database for this prototype
     * but in later build, it will be calculated through current position and position
     * of the promotion based on long and lat.
     */
    private static final String PROMOTIONTABLE_CREATE =
        "create table promotions (_id integer primary key autoincrement, "
        + "type text not null, key_word text not null, description text not null, "
        + "location text not null, dis double not null, store_name text not null, "
        + "up_vote integer not null, total_vote integer not null, vote_ratio double, lat double, longi double);";

    private static final String PREFERENCETABLE_CREATE =
    	"create table preference (_id integer primary key autoincrement, "
    	+ "radius double, promotion_per_store integer, vote_ratio double, "
    	+ "Clothing boolean, Convenient boolean, Entertainment boolean, Electronics boolean, "
    	+ "Restaurant boolean, Others boolean);";
    
    private static final String DATABASE_NAME = "realdealdata";
    private static final String DATABASE_TABLE = "promotions";
    private static final String PREFERENCE_TABLE = "preference";
    private static final int DATABASE_VERSION = 2;

    private SQLiteDatabase mDb;
    private final Context mCtx;

    /**
     * Constructor - takes the context to allow the database to be opened/created
     * @param ctx the Context within which to work
     */
    public RealDealDbAdapter(Context ctx) 
    {
        this.mCtx = ctx;
    }
    
    /**
     * Open the notes database. If it cannot be opened, try to create a new instance of
     * the database. If it cannot be created, throw an exception to signal the failure
     * @return this (self reference, allowing this to be chained in an initialization call)
     * @throws SQLException if the database could be neither opened or created
     */
    public RealDealDbAdapter open() throws SQLException 
    {
        try 
        {
            mDb = mCtx.openDatabase(DATABASE_NAME, null);
        } 
        catch (FileNotFoundException e) 
        {
            try 
            {
                mDb = mCtx.createDatabase(DATABASE_NAME, DATABASE_VERSION, 0, null);
                mDb.execSQL(PROMOTIONTABLE_CREATE);
                mDb.execSQL(PREFERENCETABLE_CREATE);
                //set default user preference
                createPreference();
                //insert sample data
                populateData();
            } 
            catch (FileNotFoundException e1) 
            {
                throw new SQLException("Could not create database");
            }
        }
        return this;
    }

    public void close() 
    {
        mDb.close();
    }

    /**
     * Create a new note using the title and body provided. If the note is successfully created
     * return the new rowId for that note, otherwise return a -1 to indicate failure.
     * @param title the title of the note
     * @param body the body of the note
     * @return rowId or -1 if failed
     */
    public long createPromotion(String type, String keyword, String description, String location, String storeName) 
    {
        ContentValues initialValues = new ContentValues();

        double rLat = generator.nextDouble() * 0.04 + RealDealPrototypeActivity.getCurrentLocation()[0];
    	double rLongi = generator.nextDouble() * 0.04 + RealDealPrototypeActivity.getCurrentLocation()[1];
        double[] toLoc = new double[2];
        toLoc[0] = rLat;
        toLoc[1] = rLongi;
    	double dis;
        
        dis = findDis(RealDealPrototypeActivity.getCurrentLocation(), toLoc);
        
        initialValues.put(KEY_TYPE, type);
        initialValues.put(KEY_KEYWORD, keyword);
        initialValues.put(KEY_DESCRIPTION, description);
        initialValues.put(KEY_LOCATION, location);
        initialValues.put(KEY_STORENAME, storeName);
        initialValues.put(KEY_UPVOTE, 0);
        initialValues.put(KEY_TOTALVOTE, 0);
        initialValues.put(KEY_LAT, rLat);
        initialValues.put(KEY_LONGI, rLongi);
        initialValues.put(KEY_DIS, dis);
        return mDb.insert(DATABASE_TABLE, null, initialValues);
    }
    
    public double findDis(double[] from, double[] to)
    {
    	double dis;
    	
    	//double PI_DIV_RAD = 0.0174;
    	//double KMS_PER_MILE = 1.609;
    	double EARTH_RADIUS_IN_MILES = 3963.19;
    	//double EARTH_RADIUS_IN_KMS = EARTH_RADIUS_IN_MILES * KMS_PER_MILE;
    	//double MILES_PER_LATITUDE_DEGREE = 69.1;
    	//double KMS_PER_LATITUDE_DEGREE = MILES_PER_LATITUDE_DEGREE * KMS_PER_MILE;
    	//double LATITUDE_DEGREES = EARTH_RADIUS_IN_MILES / MILES_PER_LATITUDE_DEGREE;
    	
    	dis = EARTH_RADIUS_IN_MILES * Math.acos( Math.sin(deg2rad(from[0])) * Math.sin(deg2rad(to[0])) + 
                Math.cos(deg2rad(from[0])) * Math.cos(deg2rad(to[0])) * 
                Math.cos(deg2rad(to[1]) - deg2rad(from[1]))); 
    	
    	return dis;
    }
    
    public double deg2rad (double degrees)
    {
    	double radius;
    	radius = degrees / 180.0 * java.lang.StrictMath.PI;
    	return radius;
    }

    /**
     * set the user preference by updating the corresponding table in DB
     **/
    public long createPreference()
    {
    	ContentValues initialValues = new ContentValues();
                
        initialValues.put(KEY_RADIUS, -1);
        initialValues.put(KEY_PROMOTION_PER_STORE, -1);
        initialValues.put(KEY_VOTERATIO, -1);
        initialValues.put("Clothing", true);
        initialValues.put("Convenient", true);
        initialValues.put("Entertainment", true);
        initialValues.put("Electronics", true);
        initialValues.put("Restaurant", true);
        initialValues.put("Others", true);
        
        return mDb.insert(PREFERENCE_TABLE, null, initialValues);
    }
    
    /**
     * set the user preference by updating the corresponding table in DB
     **/
    public boolean setPreference(double radius, int promotion_per_store, double voteRatio, Bundle types)
    {
    	ContentValues args = new ContentValues();
    	
    	if (radius >= 0)
    	{
            args.put(KEY_RADIUS, radius);
    	}
        if (promotion_per_store >= 1)
        {
            args.put(KEY_PROMOTION_PER_STORE, promotion_per_store);
        }
        if (voteRatio >= 0 && voteRatio <= 100)
        {
        	args.put(KEY_VOTERATIO, voteRatio);
        }
        args.put("Clothing", types.getBoolean("Clothing", true));
        args.put("Convenient", types.getBoolean("Convenient", true));
        args.put("Entertainment", types.getBoolean("Entertainment", true));
        args.put("Electronics", types.getBoolean("Electronics", true));
        args.put("Restaurant", types.getBoolean("Restaurant", true));
        args.put("Others", types.getBoolean("Others", true));
    	
    	return mDb.update(PREFERENCE_TABLE, args, KEY_ROWID + "=1", null) > 0;
    }
    
    public Cursor fetchPreference() throws SQLException 
    {
        Cursor result = mDb.query(true, PREFERENCE_TABLE, new String[] {
        		KEY_ROWID, KEY_RADIUS, KEY_PROMOTION_PER_STORE, KEY_VOTERATIO, "Clothing", "Convenient", "Entertainment", "Electronics", "Restaurant", "Others"}, KEY_ROWID + "=1", null, null,
                null, null);
        if ((result.count() == 0) || !result.first()) 
        {
            throw new SQLException("No promotion matching ID: 1");
        }
        return result;
    }
    
    /**
     * Return a Cursor positioned at the first promotion that matches the specified fields
     */
    public Cursor fetchPreferredPromotions(double radius, int promotion_per_store, double voteRatio, Bundle types) throws SQLException 
    {
    	String whereClause = new String();
    	String groupbyClause = new String();
    	Cursor result;
    	boolean notFirstType = false;
    	
    	if (radius >= 0)
    	{
    		String tempDis = new Double(radius).toString();
    		whereClause += KEY_DIS + "<='" + tempDis+"'";
    	}
    	if (voteRatio > 0)
    	{
    		if(whereClause.length() > 0)
    		{
        		whereClause += " and ";
    		}
    		String tempVoteRatio = Double.toString(voteRatio);
    		whereClause += KEY_TOTALVOTE + "<>0 and " + KEY_VOTERATIO + ">='" + tempVoteRatio + "'";
    	}
    	if(whereClause.length() > 0)
		{
    		whereClause += " and ";
		}
    	whereClause += "(";
    	if(types.getBoolean("Clothing", true))
    	{
    		whereClause += KEY_TYPE + "='Clothing'";
    		notFirstType = true;
    	}
        if(types.getBoolean("Convenient", true))
        {
        	if(notFirstType)
        	{
        		whereClause += " OR ";
        	}
        	else
        	{
        		notFirstType = true;
        	}
        	whereClause += KEY_TYPE + "='Convenient'";
        }
        if(types.getBoolean("Entertainment", true))
        {
        	if(notFirstType)
        	{
        		whereClause += " OR ";
        	}
        	else
        	{
        		notFirstType = true;
        	}
        	whereClause += KEY_TYPE + "='Entertainment'";
        }
        if(types.getBoolean("Electronics", true))
        {
        	if(notFirstType)
        	{
        		whereClause += " OR ";
        	}
        	else
        	{
        		notFirstType = true;
        	}
        	whereClause += KEY_TYPE + "='Electronics'";
        }
        if(types.getBoolean("Restaurant", true))
        {
        	if(notFirstType)
        	{
        		whereClause += " OR ";
        	}
        	else
        	{
        		notFirstType = true;
        	}
        	whereClause += KEY_TYPE + "='Restaurant'";
        }
        if(types.getBoolean("Others", true))
        {
        	if(notFirstType)
        	{
        		whereClause += " OR ";
        	}
        	else
        	{
        		notFirstType = true;
        	}
        	whereClause += KEY_TYPE + "='Others'";
        }
        whereClause += ")";
                
        result = mDb.query(true, DATABASE_TABLE, new String[] {
        		KEY_ROWID, KEY_TYPE, KEY_KEYWORD, KEY_DESCRIPTION, KEY_LOCATION, KEY_DIS, KEY_STORENAME, KEY_UPVOTE, KEY_TOTALVOTE, KEY_VOTERATIO, KEY_LAT, KEY_LONGI}, whereClause, null, null,
                null, KEY_DIS);
        
        /*
        if(promotion_per_store >= 1)
        {
        	groupbyClause += KEY_STORENAME + ", " + KEY_DIS + " LIMIT " + Integer.toString(promotion_per_store);
        	
        	result = mDb.query(true, DATABASE_TABLE, new String[] {
            		KEY_ROWID, KEY_TYPE, KEY_KEYWORD, KEY_DESCRIPTION, KEY_LOCATION, KEY_DIS, KEY_STORENAME, KEY_UPVOTE, KEY_TOTALVOTE, KEY_VOTERATIO, KEY_LAT, KEY_LONGI}, whereClause, null, groupbyClause,
                    null, KEY_DIS);
        }
        else
        {
        	result = mDb.query(true, DATABASE_TABLE, new String[] {
	        		KEY_ROWID, KEY_TYPE, KEY_KEYWORD, KEY_DESCRIPTION, KEY_LOCATION, KEY_DIS, KEY_STORENAME, KEY_UPVOTE, KEY_TOTALVOTE, KEY_VOTERATIO, KEY_LAT, KEY_LONGI}, whereClause, null, null,
	                null, KEY_DIS);
        }
        */
        
        return result;
    }
    
    /**
     * Delete the note with the given rowId
     * @param rowId id of note to delete
     * @return true if deleted, false otherwise
     */
    public boolean deleteNote(long rowId) 
    {
        return mDb.delete(DATABASE_TABLE, KEY_ROWID + "=" + rowId, null) > 0;
    }

    /**
     * Return a Cursor over the list of all notes in the database
     * @return Cursor over all notes
     */
    public Cursor fetchAllPromotions() 
    {
    	
        return mDb.query(DATABASE_TABLE, new String[] {
                KEY_ROWID, KEY_TYPE, KEY_KEYWORD, KEY_DESCRIPTION, KEY_LOCATION, KEY_DIS, KEY_STORENAME, KEY_UPVOTE, KEY_TOTALVOTE, KEY_VOTERATIO, KEY_LAT, KEY_LONGI}, null, null, null, null, KEY_DIS);
    }

    /**
     * Return a Cursor positioned at the note that matches the given rowId
     * @param rowId id of note to retrieve
     * @return Cursor positioned to matching note, if found
     * @throws SQLException if note could not be found/retrieved
     */
    public Cursor fetchPromotion(long rowId) throws SQLException 
    {
        Cursor result = mDb.query(true, DATABASE_TABLE, new String[] {
        		KEY_ROWID, KEY_TYPE, KEY_KEYWORD, KEY_DESCRIPTION, KEY_LOCATION, KEY_DIS, KEY_STORENAME, KEY_UPVOTE, KEY_TOTALVOTE, KEY_VOTERATIO, KEY_LAT, KEY_LONGI}, KEY_ROWID + "=" + rowId, null, null,
                null, null);
        if ((result.count() == 0) || !result.first()) 
        {
            throw new SQLException("No promotion matching ID: " + rowId);
        }
        return result;
    }
    
    /**
     * Return a Cursor positioned at the first promotion that matches the specified fields
     */
    public Cursor fetchMatchingPromotions(String type, String keyword, String description, String location, double dis, String storeName, double voteRatio) throws SQLException 
    {
    	String whereClause = new String();
    	if (type != null)
    	{
    		whereClause += KEY_TYPE + "='" + type+"'";
    	}
    	if (keyword != null)
    	{
    		keyword = keyword.toLowerCase();
    		if(whereClause.length() > 0)
        		whereClause += " and ";
    		whereClause += KEY_KEYWORD + " LIKE '%" + keyword+"%'";
    	}
    	if (description != null)
    	{
    		description = description.toLowerCase();
    		if(whereClause.length() > 0)
        		whereClause += " and ";
    		whereClause += KEY_DESCRIPTION + " LIKE '%" + description+"%'";
    	}
    	if (location != null)
    	{
    		location = location.toLowerCase();
    		if(whereClause.length() > 0)
        		whereClause += " and ";
    		whereClause += KEY_LOCATION + " LIKE '%" + location+"%'";
    	}
    	if (dis >= 0)
    	{
    		if(whereClause.length() > 0)
        		whereClause += " and ";
    		String tempDis = new Double(dis).toString();
    		whereClause += KEY_DIS + "<='" + tempDis+"'";
    	}
    	if (storeName != null)
    	{
    		storeName = storeName.toLowerCase();
    		if(whereClause.length() > 0)
        		whereClause += " and ";
    		whereClause += KEY_STORENAME + " LIKE '%" + storeName+"%'";
    	}
    	if (voteRatio > 0)
    	{
    		if(whereClause.length() > 0)
        		whereClause += " and ";
    		String tempVoteRatio = Double.toString(voteRatio);
    		whereClause += KEY_TOTALVOTE + "<>0 and " + KEY_VOTERATIO + ">='" + tempVoteRatio + "'";
    	}
    	    	    	
        Cursor result = mDb.query(true, DATABASE_TABLE, new String[] {
        		KEY_ROWID, KEY_TYPE, KEY_KEYWORD, KEY_DESCRIPTION, KEY_LOCATION, KEY_DIS, KEY_STORENAME, KEY_UPVOTE, KEY_TOTALVOTE, KEY_VOTERATIO, KEY_LAT, KEY_LONGI}, whereClause, null, null,
                null, KEY_DIS);
        
        return result;
    }

    /**
     * Update the note using the details provided. The note to be updated is specified using
     * the rowId, and it is altered to use the title and body values passed in
     * @param rowId id of note to update
     * @param title value to set note title to
     * @param body value to set note body to
     * @return true if the note was successfully updated, false otherwise
     */
    public boolean updatePromotion(long rowId, String type, String keyword, String description, String location, double dis, String storeName, int upVote, int totalVote) 
    {
        ContentValues args = new ContentValues();
        if (type != null)
        args.put(KEY_TYPE, type);
        if (keyword != null)
        args.put(KEY_KEYWORD, keyword);
        if (description != null)
        args.put(KEY_DESCRIPTION, description);
        if (location != null)
        {
        	args.put(KEY_LOCATION, location);
        	double rLat = generator.nextDouble() * 0.04 + RealDealPrototypeActivity.getCurrentLocation()[0];
        	double rLongi = generator.nextDouble() * 0.04 + RealDealPrototypeActivity.getCurrentLocation()[1];
        	args.put(KEY_LAT, rLat);
        	args.put(KEY_LONGI, rLongi);
        }
        if (dis >= 0)
        args.put(KEY_DIS, dis);
        if (storeName != null)
        args.put(KEY_STORENAME, storeName);
        if (upVote >= 0)
        args.put(KEY_UPVOTE, upVote);
        if (totalVote >= 0)
        {
        	args.put(KEY_TOTALVOTE, totalVote);
        	double voteRatio = (double)upVote / (double)totalVote;
        	args.put(KEY_VOTERATIO, voteRatio);
        }
        return mDb.update(DATABASE_TABLE, args, KEY_ROWID + "=" + rowId, null) > 0;
    }
    
    /**
     * populate database with sample promotions
     **/
    private void populateData() 
    {
    	createPromotion("Restaurant", "free drink", "free drink with any purchase. blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah blah", "20 Allston Way", "Sub Way");
    	createPromotion("Clothing", "buy 1 get 1 free", "fancy new cloth with low price", "1900 Shattuck Way", "Fancy World");
    	createPromotion("Electronics", "bonus trade in", "trade in games now counts for more, limited offer", "345 Telegraph", "EC Games");
    	createPromotion("Entertainment", "free drink", "get free drink with a movie ticket package", "1330 Shattuck Way", "AMW");
    	createPromotion("Restaurant", "free wifi", "free wifi connection with any purchase in store", "500 Oxford St", "MoonBucks");
    	createPromotion("Clothing", "clothing price cut", "off season clothing price cut", "620 Bancroft Way", "MV");
    	createPromotion("Others", "low price pet food", "limited offer, low price pet food", "1220 Addison St", "PetMart");
    	createPromotion("Electronics", "printer refill", "low price printer ink refill", "1145 Center St", "Fry");
    	createPromotion("Restaurant", "pet welcome", "new pet cafe grand opening, pet welcome", "2110 University Ave", "Pet Bar");
    	createPromotion("Restaurant", "free beer", "celebrate championship, limited time free beer party", "365 Hearst Ave", "Bear's Lair");
    	createPromotion("Convenient", "snacks on sale", "all varity snack packs are on sale for limited time", "12 Allston Way", "SafeWay");
    }
 }
